"use server";

import { getSummaries, getTags } from "../action";
import { titleFont } from "@/components/fonts";
import { notFound } from "next/navigation";
import React from "react";
import PostSummary from "./PostSummary";
import Pagination from "./Pagination";
import TagOverview from "./TagOverview";
import { isLoggedIn } from "@/components/auth";

export default async function Blog({
  params,
  searchParams,
}: {
  params: Promise<{ tag: string[] | undefined }>;
  searchParams: Promise<{ page: string | undefined }>;
}) {
  const pageNumber = await getPageNumber(searchParams);
  const { tag } = await params;
  const currentTag = tag != null && tag.length >= 1 ? tag[0] : null;
  if ((tag?.length ?? 0) > 1) {
    notFound();
  }

  const { metadata, numberOfPages } = await getSummaries(
    pageNumber,
    currentTag
  );
  if (pageNumber > numberOfPages) {
    notFound();
  }
  const loggedIn = await isLoggedIn();
  const tags = await getTags(loggedIn);

  return (
    <>
      <div
        style={{
          display: "flex",
          flexDirection: "column",
        }}
        className={titleFont.className}
      >
        <span style={{ fontSize: 18 }}>naresh writes</span>
        <span style={{ fontSize: 12 }}>...occasionally</span>
      </div>
      <TagOverview tags={tags} currentTag={currentTag} />
      {metadata
        .filter((m) => loggedIn || !m.is_draft)
        .map((m) => (
          <PostSummary metadata={m} key={m.slug} loggedIn={loggedIn} />
        ))}
      <Pagination numberOfPages={numberOfPages} pageNumber={pageNumber} />
    </>
  );
}

async function getPageNumber(
  searchParams: Promise<{ page: string | undefined }>
) {
  const { page } = await searchParams;
  const result = page ? parseInt(page) : 1;
  if (isNaN(result) || result < 1) {
    notFound();
  }
  return result;
}
