import MarkdownIt from "markdown-it";
import hljs from "highlight.js";
import "highlight.js/styles/github-dark.css";
import { KeyboardEvent } from "react";

const md = new MarkdownIt({
  linkify: true,
  typographer: true,
  highlight: (str, lang) => {
    if (lang && hljs.getLanguage(lang)) {
      try {
        return hljs.highlight(str, { language: lang }).value;
      } catch {}
    }
    return "";
  },
});

export default function ContentArea({
  content,
  setContent,
}: {
  content: string;
  setContent: (e: string) => void;
}) {
  const renderedContent =
    content.trim() != ""
      ? { __html: md.render(content) }
      : { __html: "content is previewed here!" };
  return (
    <div
      style={{
        display: "flex",
        flexDirection: "column",
        gap: 8,
        fontSize: 14,
        marginTop: "1rem",
      }}
    >
      <span>content:</span>
      <textarea
        style={{
          resize: "none",
          width: "100%",
          height: "10rem",
          background: "#333",
          borderStyle: "none",
          color: "#eee",
        }}
        onChange={(e) => setContent(e.target.value)}
        onKeyDown={(e) => {
          handleTabbing(e);
          setContent(e.currentTarget.value);
        }}
        value={content}
      />
      <div
        style={{
          border: "1px solid #999",
          width: "100%",
          padding: "0.5rem",
          boxSizing: "border-box",
          fontSize: "0.8rem",
        }}
        dangerouslySetInnerHTML={renderedContent}
      ></div>
    </div>
  );
}

function handleTabbing(e: KeyboardEvent<HTMLTextAreaElement>) {
  if (e.key == "Tab") {
    e.preventDefault();
    const start = e.currentTarget.selectionStart;
    const end = e.currentTarget.selectionEnd;
    const value = e.currentTarget.value;
    if (e.shiftKey) {
      const currentLineStart = value.lastIndexOf("\n", start - 1) + 1;
      const currentLineEnd = value.indexOf("\n", start);
      const currentLine = value.substring(
        currentLineStart,
        currentLineEnd === -1 ? value.length : currentLineEnd
      );
      const numSpaces = currentLine.search(/\S|$/);
      let newSpaces = numSpaces;
      if (numSpaces % 4 === 0) {
        newSpaces = Math.max(0, numSpaces - 4);
      } else {
        newSpaces = Math.max(0, numSpaces - (numSpaces % 4));
      }
      const newLine = " ".repeat(newSpaces) + currentLine.trimStart();
      const newValue =
        value.substring(0, currentLineStart) +
        newLine +
        value.substring(currentLineEnd === -1 ? value.length : currentLineEnd);
      e.currentTarget.value = newValue;
      e.currentTarget.selectionStart = e.currentTarget.selectionEnd =
        currentLineStart + newSpaces;
    } else {
      e.currentTarget.value =
        value.substring(0, start) + "    " + value.substring(end, value.length);
      e.currentTarget.selectionStart = e.currentTarget.selectionEnd = start + 4;
    }
  } else if (e.key == "Enter") {
    e.preventDefault();
    const start = e.currentTarget.selectionStart;
    const value = e.currentTarget.value;
    const beforeCursor = value.substring(0, start);
    const afterCursor = value.substring(start);
    const lastLineStart = beforeCursor.lastIndexOf("\n") + 1;
    const currentLine = beforeCursor.substring(lastLineStart);
    const indentMatch = currentLine.match(/^(\s*)/);
    const indent = indentMatch ? indentMatch[1] : "";

    // Add new line with same indentation
    e.currentTarget.value = beforeCursor + "\n" + indent + afterCursor;
    e.currentTarget.selectionStart = e.currentTarget.selectionEnd =
      start + 1 + indent.length;
  }
}
